package com.indiscale.fdo.manager.service.mock;

import com.indiscale.fdo.DelegatorFdo;
import com.indiscale.fdo.api.FDO;
import com.indiscale.fdo.api.FdoComponent;
import com.indiscale.fdo.api.FdoUpdate;
import com.indiscale.fdo.api.RepositoryConfig;
import com.indiscale.fdo.api.RepositoryConnection;
import com.indiscale.fdo.api.RepositoryConnectionFactory;
import com.indiscale.fdo.api.RepositoryType;
import com.indiscale.fdo.api.TokenAuthenticationInfo;
import com.indiscale.fdo.api.ValidationException;
import java.util.UUID;

class MockRepository implements RepositoryConnection {

  public static class MockFDO extends DelegatorFdo {

    private final String pid;

    public MockFDO(FDO fdo) {
      super(fdo);
      pid = "some.prefix/" + UUID.randomUUID().toString();
    }

    @Override
    public String getPID() {
      return pid;
    }
  }

  private final String id;
  private final RepositoryType type;

  public MockRepository(RepositoryConfig config) {
    this.id = config.getId();
    this.type = config.getType();
  }

  @Override
  public String getId() {
    return id;
  }

  @Override
  public RepositoryType getType() {
    return type;
  }

  @Override
  public void close() {}

  @Override
  public boolean isOnline() {
    return true;
  }

  @Override
  public FDO createFDO(FDO fdo) throws ValidationException {
    return new MockFDO(fdo);
  }

  @Override
  public <T extends FdoComponent> T copyFdoComponent(T component) {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public FDO updateFdo(FdoUpdate fdo) {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public void deleteFdoComponent(FdoComponent oldComponent, String newPid) {
    // TODO Auto-generated method stub
  }

  @Override
  public void setTokenAuthenticationInfo(TokenAuthenticationInfo authentication) {
    // TODO Auto-generated method stub
  }
}

public class MockRepositoryFactory implements RepositoryConnectionFactory {

  static class MockRepositoryConfig implements RepositoryConfig {

    private final String id;

    public MockRepositoryConfig(String id) {
      this.id = id;
    }

    @Override
    public String getId() {
      return id;
    }

    @Override
    public RepositoryType getType() {
      return TYPE;
    }

    @Override
    public String get(String key) {
      return null;
    }

    @Override
    public String getDescription() {
      return "This is not real.";
    }

    @Override
    public String getMaintainer() {
      return "Mocking Institute of Technology";
    }
  }

  public static final RepositoryType TYPE =
      new RepositoryType("MockDOIP").description("A mock-up for a DOIP repository.");

  @Override
  public RepositoryConnection createConnection(RepositoryConfig config) {
    return new MockRepository(config);
  }

  @Override
  public RepositoryType getType() {
    return TYPE;
  }

  public RepositoryConfig createMockConfig(String id) {
    return new MockRepositoryConfig(id);
  }
}
